angular.module('soundDebugger', ['ngAnimate', 'ui.bootstrap'])
	.filter("toArray", function () {
		return function (obj) {
			var result = [];
			angular.forEach(obj, function (val, key) {
				result.push(val);
			});
			return result;
		};
	});

var DeepMerge = function (target, newValues) {
	for (var id in newValues) {
		var rhs = newValues[id];
		if (target[id] instanceof Object && rhs instanceof Object) {
			target[id] = rhs;
		}
		else {
			target[id] = rhs;
		}
	}
};

var hbPageDebugRoutines = {
	scope: null
};

var ButtonsCtrl = function ($scope) {
	hbPageDebugRoutines.scope = $scope;
	$scope.spawnedMixers = 0;
	$scope.spawnedVoices = 0;

	$scope.voiceSortFunc = function (item) {
		console.log(item);
		return item.startTime;
	}

	$scope.showLog = false;
	$scope.showLoopPoints = true;
	$scope.showStreaming = true;

	$scope.newSound = function (parentID, isMixer) {

		var name;
		var spawnIdx;
		if (isMixer) {
			spawnIdx = $scope.spawnedMixers = $scope.spawnedMixers + 1;
			name = "mix" + spawnIdx;
		}
		else {
			spawnIdx = $scope.spawnedVoices = $scope.spawnedVoices + 1;
			name = "voice" + spawnIdx;
		}

		var sound = $scope.nodeStructure.voices[name];
		if (sound == undefined || sound == null) {
			sound = new Object();
			$scope.nodeStructure.voices[name] = sound;
			sound.name = "bobby " + spawnIdx;
		}
		sound.isMixer = isMixer;
		sound.parent = parentID;
		sound.volume = 0.5;
		$scope.updateActiveCount();
	}

	$scope.removeVoice = function (id) {
		delete $scope.nodeStructure.voices[id];
		$scope.updateActiveCount();
	}

	$scope.addVoices = function (voiceList) {
		for (var id in voiceList) {
			var newVoice = voiceList[id];
			$scope.nodeStructure.voices[id] = voiceList[id];
			if (newVoice.mixerPtr == null) {
				$scope.nodeStructure.voiceHistory.push({ id: id, name: newVoice.name, startTime: newVoice.startTime });
			}
		}
		$scope.updateActiveCount();
	}

	$scope.updateVoices = function (voiceList) {
		for (var id in voiceList) {
			DeepMerge($scope.nodeStructure.voices[id], voiceList[id]);
		}
		$scope.updateActiveCount();
	}

	$scope.updateActiveCount = function () {
		$scope.nodeStructure.activeVoices.curr = 0;
		$scope.nodeStructure.streamingVoices.curr = 0;

		for (var id in $scope.nodeStructure.voices) {
			var voice = $scope.nodeStructure.voices[id];
			if (voice.mixerPtr == undefined) {
				$scope.nodeStructure.activeVoices.curr = $scope.nodeStructure.activeVoices.curr + 1;
				if (voice.streamed) {
					$scope.nodeStructure.streamingVoices.curr = $scope.nodeStructure.streamingVoices.curr + 1;
				}
			}
		}
		$scope.nodeStructure.activeVoices.max = Math.max($scope.nodeStructure.activeVoices.curr, $scope.nodeStructure.activeVoices.max);
		$scope.nodeStructure.streamingVoices.max = Math.max($scope.nodeStructure.streamingVoices.curr, $scope.nodeStructure.streamingVoices.max);
	}

	$scope.socketStateDescriptions = [
		"Connecting...",
		"",
		"Closing Connection",
		"Connection Closed",
	];

	$scope.socketStatusReason = "";

	var GetHost = function () {
		if (document.location.host != "")
			return document.location.host;
		return "localhost";
	}

	var connectHandler = function (event) {
		if (event.reason instanceof String) {
			$scope.socketStatusReason = ": " + event.reason;
		}
		else {
			$scope.socketStatusReason = "";
		}
		$scope.$apply();
	};

	$scope.socket = new WebSocket("ws://" + GetHost() + "/websocket/MortarAudioMixer");
	$scope.socket.onclose = connectHandler;
	$scope.socket.onopen = connectHandler;

	$scope.socket.onmessage = function (msg) {
		var msgObj;
		try {
			msgObj = JSON.parse(msg.data);
		}
		catch (err) {
			return;
		}

		if (msgObj == null) {
			return;
		}

		if (msgObj.newVoice != null) {
			$scope.addVoices(msgObj.newVoice);
		}
		if (msgObj.newMixer != null) {
			$scope.addVoices(msgObj.newMixer);
		}
		if (msgObj.stateChange != null) {
			$scope.updateVoices(msgObj.stateChange);
		}
		if (msgObj.removed != null) {
			$scope.removeVoice(msgObj.removed);
		}

		$scope.lastMessage = msgObj;
		$scope.$apply();
	}

	$scope.fadeoutTimeText = 1.0;
	$scope.origAnimTimeout = animStyles.innerHTML;
	$scope.setAnimationTimeout = function (time) {
		time = parseFloat(time);
		if (isNaN(time) || !isFinite(time)) {
			$scope.fadeoutTimeText = "2";
			return;
		}
		if (time < 0.01)
			time = 0.01;
		animStyles.innerHTML = $scope.origAnimTimeout.replace(/1s/g, time + "s");
	}


	$scope.lastMessage = {};

	$scope.nodeStructure = {
		activeVoices: { curr: 0, max: 0 },
		streamingVoices: { curr: 0, max: 0 },
		voices: {},
		voiceHistory: []
	};
};
/*
hbPageDebugRoutines.setupDummyData = function () {
	hbPageDebugRoutines.scope.nodeStructure = {
		// add testing data here!
	};
	hbPageDebugRoutines.scope.updateActiveCount();
	hbPageDebugRoutines.scope.$apply();
}

hbPageDebugRoutines.removeItem = function (id) {
	hbPageDebugRoutines.scope.removeVoice(id);
	hbPageDebugRoutines.scope.$apply();
}

hbPageDebugRoutines.createItem = function (newID, parentID) {
	var newVoices = new Object();
	newVoices[newID] = {
		"isPlaying": true,
		"name": "TestSound_" + newID,
		"parent": parentID,
		"pitchShift": { "rate": 0, "startFrame": 0, "startValue": 1, "targetValue": 0 },
		"volume": { "rate": 0, "startFrame": 0, "startValue": 1, "targetValue": 0 }
	};
	hbPageDebugRoutines.scope.addVoices(newVoices);
	hbPageDebugRoutines.scope.$apply();
}

hbPageDebugRoutines.testFadeItem = function () {
	hbPageDebugRoutines.createItem("FadeItem", hbPageDebugRoutines.scope.nodeStructure.voices["0x00000000"].mixerPtr);
	setTimeout(function () {
		hbPageDebugRoutines.removeItem("FadeItem");
		setTimeout(hbPageDebugRoutines.testFadeItem, 2000);
	}, 3000);
}

var runTests = function () {
	hbPageDebugRoutines.setupDummyData();
	hbPageDebugRoutines.testFadeItem();
}
*/
