precision lowp int; 
precision lowp float;

uniform sampler2D DiffuseMap;

uniform lowp float wrappedTime;

varying lowp vec2 uvs;
varying lowp vec3 worldnormal;
varying lowp vec4 vertexcolour;
varying lowp float ypos;

const float worldToScreenScale = 500.0;

//implementation from the bible of Pete McNeill - praise be
vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}
vec3 hsv2rgb(vec3 c)
{
	vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
	return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() 
{
	vec4 colour = texture2D(DiffuseMap, uvs);
	vec3 hsvColour = rgb2hsv(colour.rgb);
	
	hsvColour.x += wrappedTime + ypos / worldToScreenScale;
	hsvColour.y += 0.2;
	
	colour.rgb = hsv2rgb(hsvColour);
	gl_FragColor = colour;
}
