precision lowp int; 
precision lowp float;

uniform sampler2D DiffuseMap;

uniform lowp vec2 screenSize;

varying lowp vec4 uvs;
varying lowp vec3 worldnormal;
varying lowp vec4 vertexcolour;

const vec2 downsampledSize = vec2(200, 400);
const float aspectY = 3.0 / 2.0;
const float barrelPower = 1.15;
const float barrelSize = 1.05;

//these are UV offsets
// dont use pixel offsets so it scales nicely
const vec2 redOffset = vec2((1.0 / downsampledSize.x) * -1.0, 0.0);
const vec2 blueOffset = vec2((1.0 / downsampledSize.x) * 1.0, 0.0);
const vec2 greenOffset = vec2(0.0, (1.0 / downsampledSize.y) * 1.0);

vec2 distort(vec2 p)
{	
	p = p * 2.0 - 1.0;
    float theta  = atan(p.y, p.x);
    float radius = length(p);
    
	//transform power curve using mix to make it curve more at the edges
	radius = mix(radius, pow(radius, barrelPower), radius);
    
	p.x = radius * cos(theta) * barrelSize;
    p.y = radius * sin(theta) * barrelSize;
    return 0.5 * (p + 1.0);
}

float calcEdgeFade(vec2 p)
{
	p = p * vec2(2.0) - vec2(1.0);
	return 1.0 - clamp(pow(pow(abs(p.x), 10.0) + pow(abs(p.y), 10.0), 8.0), 0.0, 1.0);
}

void main() 
{
	vec2 pixelPos = uvs.xy;
	
	//barrel out the pixel pos to simulate CRT monitor shape
	pixelPos = distort(pixelPos);
	
	//fade this pixel as we get to the edge of the screen
	float edgeFade = calcEdgeFade(pixelPos);
	
	//clamp the pixel down to simulate downsampling
	pixelPos.x = floor(pixelPos.x * downsampledSize.x) / downsampledSize.x;
	pixelPos.y = floor(pixelPos.y * downsampledSize.y) / downsampledSize.y;
	
	float r = edgeFade * texture2D(DiffuseMap, vec2(pixelPos.x, pixelPos.y) + redOffset).r;
	float g = edgeFade * texture2D(DiffuseMap, vec2(pixelPos.x, pixelPos.y) + greenOffset).g;
	float b = edgeFade * texture2D(DiffuseMap, vec2(pixelPos.x, pixelPos.y) + blueOffset).b;
	float a = 1.0;
	gl_FragColor = vec4(r, g, b, a);
	
	//uncomment the line below to disable the effect
	//gl_FragColor = texture2D(DiffuseMap, vec2(uvs.x, uvs.y));
}
